/* 
 * Cryosphere Mud Client version 0.1
 * Copyright (C) 1998 Abigail Brady
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 */

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>
#include <errno.h>
#include <fcntl.h>

#include "Socket.h"
//#include "log.h"
//#include "privs.h"
//#include "musicio.h"
//#include "pflags.h"

Socket::Socket() {
  this->port = -1;
  buffer = "";
  fd = -1;
}

Socket::Socket(int port) {
  this->port = port;
  buffer="";
  listener = true;
  fd = socket(AF_INET, SOCK_STREAM, 0);
  //  string why = "TCP4 port ";
  //  why += port;
  //  char why[200];
  //  sprintf(why, "TCP4 port %i", port);
  //  xreg(fd, why);
  //  xselect(fd);
  int yes = 1;
  setsockopt(fd, SOL_SOCKET, SO_REUSEADDR, &yes, sizeof yes);
  struct sockaddr_in addr;
  addr.sin_family = AF_INET;
  addr.sin_port = htons(port);
  memset(&addr.sin_addr, 0, sizeof addr.sin_addr);

  if (bind(fd, (struct sockaddr *)&addr, sizeof addr)!=0) {
    //      log(PFL_SEEINFO, 0, "socket", "port %i in use : not accepting connections", port);
      dead = true;
      return;
  };

  listen(fd, 10);
  fcntl(fd, F_SETFL, O_NONBLOCK);
  dead = false;
}

int Socket::connect(int fd) {
    this->port = -1;
    this->fd = fd;
    this->dead = 0;
    socklen_t length = sizeof (this->address);
    getpeername(fd, (struct sockaddr *)&this->address, &length);
    return 0;
}

int Socket::connect(const char *where, int port) {
    this->port = -1;
    struct sockaddr_in addr;
    addr.sin_family = AF_INET;
    addr.sin_port = htons(port);
    inet_aton(where, &addr.sin_addr);
    
    fd = socket(AF_INET, SOCK_STREAM, 0);
    //    xreg(fd, "outgoing connection");
    //    xselect(fd);
    int value = ::connect(fd, (struct sockaddr *)&addr, sizeof addr);
    
    int er = errno;
    
    if (value == -1) {
      //	xunreg(fd);
	::close(fd);
	return -er;
    }
    
    fcntl(fd, F_SETFL, O_NONBLOCK);
    dead = false;
    
    return true;
}


Socket *Socket::accept() {
  int sockfd;
  struct sockaddr_in addr;
  socklen_t size = sizeof addr;

  sockfd = ::accept(fd, (struct sockaddr *)&addr, &size);

  if (sockfd != -1) {

    Socket *sock = new Socket();
    sock->fd = sockfd;
    //    xreg(sockfd, "incoming connection");
    //    xselect(sockfd);
    fcntl (sockfd, F_SETFL, O_NONBLOCK);
    sock->address = addr;
    sock->dead = false;
    return sock;

  }

  return 0;

}

int Socket::read(char *data, int size) {
  if (dead) {
      errno = ECONNRESET;
      return -1;
  }
  int rval = ::read(fd, data, size);
  if (rval == -1) {
      if (errno == EAGAIN || errno == EWOULDBLOCK) {
	  return -1;
      }
      dead = true;
  }
  return rval;
}

int Socket::write(const char *data, int size) {
    if (dead) {
	errno = ECONNRESET;
	return -1;
    }
    int rval = ::write(fd, data, size);
    if (rval == -1) {
	dead = true;
    }
    return rval;
}

Socket::~Socket() {
  if (!dead) {
    shutdown(fd, 2);
    ::close(fd);
  }
  //  xunreg(fd);
}

void Socket::close() {
  shutdown(fd, 2);
  ::close(fd);
  dead = true;
}

int Socket::read_data(char *where) {
  if (dead) {
      return -1;
  }
  char a;
  int l=0;
  while (read(&a, 1)==1) {
	 buffer+=a;
	  l++;
	  if (!a) {
		  strcpy(where, buffer.c_str());
		  buffer = "";
		  return 0;
	  }
  }
  if (dead) {
      return -1;
  }
  *where = 0;
  if (l) printf("partial length : %i\n", l);
  return 0;
}
